package net.wanji.opt.servicev2.report.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.common.enums.CrossDirEnum;
import net.wanji.opt.common.enums.CrossTypeEnum;
import net.wanji.opt.common.enums.LaneTurnEnum;
import net.wanji.opt.controllerv2.report.vo.CrossRidLaneVO;
import net.wanji.opt.controllerv2.report.vo.LaneDirNumVO;
import net.wanji.opt.dao.mapper.report.CrossRidLaneMapper;
import net.wanji.opt.entity.report.CrossRidLaneDTO;
import net.wanji.opt.servicev2.report.CrossRidLaneService;
import net.wanji.opt.vo2.report.CrossRidLaneResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Service
@Slf4j
public class CrossRidLaneServiceImpl implements CrossRidLaneService {

    @Autowired
    private CrossRidLaneMapper crossRidLaneMapper;

    @Override
    public List<CrossRidLaneResult> getCrossInfoList(String crossIds) {
        List<String> crossIdList = null;
        if(ObjectUtil.isNotEmpty(crossIds)){
            crossIdList = Arrays.stream(crossIds.split(","))
                    .map(String::trim) // 去除可能的空格
                    .filter(s -> !s.isEmpty()) // 过滤掉空字符串
                    .collect(Collectors.toList());
        }
        List<CrossRidLaneDTO> crossRidLaneDTOList = crossRidLaneMapper.getCrossInfoList(crossIdList);
        Map<String, List<CrossRidLaneDTO>> crossIdMap = crossRidLaneDTOList.stream().collect(Collectors.groupingBy(CrossRidLaneDTO::getCrossId));
        List<CrossRidLaneResult> crossRidLaneResultList = new ArrayList<>();
        for (List<CrossRidLaneDTO> crossRidLaneDTOS : crossIdMap.values()) {
            Map<String, List<CrossRidLaneDTO>> laneNameMap = crossRidLaneDTOS.stream().collect(Collectors.groupingBy(x->{
                if(ObjectUtil.isNotEmpty(x.getLaneName())){
                    String[] split = x.getLaneName().split(":");
                    return split[0];
                }
                String[] strings = x.getCrossName().split("与");
                return strings[0];
            }));
            CrossRidLaneResult crossRidLaneResult = new CrossRidLaneResult();
            crossRidLaneResult.setCrossId(crossRidLaneDTOS.get(0).getCrossId());
            crossRidLaneResult.setCrossName(crossRidLaneDTOS.get(0).getCrossName());
            crossRidLaneResult.setType(CrossTypeEnum.getDesc(crossRidLaneDTOS.get(0).getType()));
            crossRidLaneResult.setLocation(getLocation(crossRidLaneDTOS.get(0).getLocation()));

            List<CrossRidLaneVO> crossRidLaneVOS = new ArrayList<>();
            for (String laneName : laneNameMap.keySet()) {
                List<CrossRidLaneDTO> laneDTOS = laneNameMap.get(laneName);
                CrossRidLaneVO crossRidLaneVO = new CrossRidLaneVO();
                crossRidLaneVO.setLaneName(laneName);
                if(ObjectUtil.isEmpty(laneDTOS.get(0).getInDir())||ObjectUtil.isEmpty(laneDTOS.get(0).getOutDir())){
                    crossRidLaneVO.setLaneDir(getLaneDir(laneDTOS.get(0).getDir()));
                }else {
                    crossRidLaneVO.setLaneDir(CrossDirEnum.getDesc(laneDTOS.get(0).getInDir())+CrossDirEnum.getDesc(laneDTOS.get(0).getOutDir())+"走向");
                }
                crossRidLaneVO.setLaneNum(laneDTOS.size());
                Map<Integer, List<CrossRidLaneDTO>> dirLaneMap = laneDTOS.stream().collect(Collectors.groupingBy(CrossRidLaneDTO::getDir));
                List<LaneDirNumVO> laneDirNumVOS = new ArrayList<>();
                for (Integer dir : dirLaneMap.keySet()) {
                    List<CrossRidLaneDTO> dirLaneDTOS = dirLaneMap.get(dir);
                    LaneDirNumVO laneDirNumVO = new LaneDirNumVO();
                    laneDirNumVO.setLaneInDir(CrossDirEnum.getDesc(dir)+"进口");
                    laneDirNumVO.setLaneDirNum(dirLaneDTOS.size());
                    Map<Integer, List<CrossRidLaneDTO>> turnMap = dirLaneDTOS.stream().collect(Collectors.groupingBy(CrossRidLaneDTO::getTurn));
                    String laneDirTurn = "";
                    for (Integer turn : turnMap.keySet()) {
                        if("".equals(laneDirTurn)){
                            laneDirTurn = turnMap.get(turn).size() + LaneTurnEnum.getDesc(turn);
                        }else {
                            laneDirTurn = laneDirTurn + "," + turnMap.get(turn).size() + LaneTurnEnum.getDesc(turn);
                        }
                    }
                    laneDirNumVO.setLaneTurnNum(laneDirTurn);
                    laneDirNumVOS.add(laneDirNumVO);
                }
                crossRidLaneVO.setLaneDirList(laneDirNumVOS);
                crossRidLaneVOS.add(crossRidLaneVO);
            }
            crossRidLaneResult.setCrossRidLaneList(crossRidLaneVOS);
            crossRidLaneResultList.add(crossRidLaneResult);
        }
        return crossRidLaneResultList;
    }

    private String getLaneDir(Integer dir) {
        switch (dir){
            case 1:
                return "南北走向";
            case 2:
                return "东北西南走向";
            case 3:
                return "东西走向";
            case 4:
                return "东南西北走向";
            case 5:
                return "南北走向";
            case 6:
                return "东北西南走向";
            case 7:
                return "东西走向";
            case 8:
                return "东南西北走向";
            default:
                return "东西走向";
        }
    }

    private String getLocation(String location) {
        // 提取经纬度
        try {
            // 去掉 "POINT(" 和 ")"
            String content = location.substring(6, location.length() - 1);

            // 按空格分割
            String[] parts = content.split("\\s+");

            if (parts.length == 2) {
                String longitude = parts[0]; // 经度
                String latitude = parts[1];  // 纬度

                return longitude + "," +latitude;
            } else {
                log.error("未找到匹配的 POINT 格式");
            }
        } catch (Exception e) {
            log.error("解析失败: " + e.getMessage());
        }
        return "";
    }
}